@@ -7,7 +7,7 @@ from avtoolbox.utils import is_posix
 from avtoolbox.utils.logger import LOGGER
 from avtoolbox.utils.yaml_parser import YAMLParser
 from avtoolbox.utils.files import search_upwards_for_file
-from avtoolbox.utils.docker import get_docker_client_binary_path, run_docker_cmd, compose_is_installed, DockerComposeClient, norm_ports, find_available_port, parse_devices, DockerException, get_docker_runtime
+from avtoolbox.utils.docker import get_docker_client_binary_path, run_docker_cmd, compose_is_installed, DockerComposeClient, norm_ports, find_available_port, DockerException
 
 # Other imports
 import yaml, os, argparse
@@ -36,7 +36,7 @@ def _merge_dictionaries(source, destination, overwrite_lists=False):
 
     return destination
 
-def _run_env(args):
+def _run_env(args, unknown_args):
     """
     Entrypoint for the `dev` command.
 
@@ -105,7 +105,7 @@ def _run_env(args):
 
     # Read the avtoolbox yml file
     custom_config = {}
-    custom_attrs = ["project", "user", "default_services", "optional_devices", "desired_runtime", "overwrite_lists"]
+    custom_attrs = ["project", "user", "default_services", "desired_runtime", "overwrite_lists", "custom_cli_arguments"]
     try:
         # Custom config
         custom_config["root"] = root
@@ -114,9 +114,8 @@ def _run_env(args):
         custom_config["uid"] = _check_avtoolbox(avtoolbox_yml, "user", "uid", default=os.getuid() if is_posix() else 1000)
         custom_config["gid"] = _check_avtoolbox(avtoolbox_yml, "user", "gid", default=os.getgid() if is_posix() else 1000)
         custom_config["default_services"] = _check_avtoolbox(avtoolbox_yml, "default_services", default=["dev"])
-        custom_config["optional_devices"] = _check_avtoolbox(avtoolbox_yml, "optional_devices", default={})
-        custom_config["runtime"] = get_docker_runtime(_check_avtoolbox(avtoolbox_yml, "desired_runtime", default="nvidia"))
         custom_config["overwrite_lists"] = _check_avtoolbox(avtoolbox_yml, "overwrite_lists", default=False)
+        custom_config["custom_cli_arguments"] = _check_avtoolbox(avtoolbox_yml, "custom_cli_arguments", default={})
 
         # Check these two attributes exist
         # Will exit if they don't
@@ -134,7 +133,7 @@ def _run_env(args):
     LOGGER.debug("Reading the default-compose.yml file...")
     default_compose_yml = os.path.realpath(os.path.join(__file__, "..", "docker", "default-compose.yml"))
     with open(default_compose_yml, "r") as f:
-        default_configs = YAMLParser(text=eval(f"f'''{f.read()}'''", globals(), custom_config)).get_data()
+        default_configs = YAMLParser(text=f.read()).get_data()
     LOGGER.debug("Finished parsing the default-compose.yml and updating values.")
 
     # Grab the default dockerignore file
@@ -144,7 +143,7 @@ def _run_env(args):
         dockerignore = f.read()
 
     LOGGER.debug("Reading existing dockerignore file...")
-    if (existing_dockerfile := search_upwards_for_file('.dockerignore')) is not None:
+    if (existing_dockerignore := search_upwards_for_file('.dockerignore')) is not None:
         with open(existing_dockerignore, "r") as f:
             dockerignore += f.read()
     LOGGER.debug("Finished reading dockerignore files.")
@@ -156,11 +155,17 @@ def _run_env(args):
     docker_compose = _merge_dictionaries(temp, default_configs, custom_config["overwrite_lists"])
 
     # If no command is passed, start up the container and attach to it
-    cmds = [args.build, args.up, args.down, args.attach] 
+    cmds = [args.build, args.up, args.down, args.attach, args.run] 
     if all(not c for c in cmds):
         args.up = True
         args.attach = True
-
+    
+    if args.run and any([args.build, args.up, args.down, args.attach]):
+        LOGGER.fatal("The '--run' command can be the only command.")
+        return
+    elif args.run and len(args.services) != 1:
+        LOGGER.fatal("The '--run' command requires only one service.")
+        return
 
     # Get the services we'll use
     if args.services is None: 
@@ -226,17 +231,10 @@ def _run_env(args):
                             LOGGER.warn(f"PORT CONFLICT: Adjusted port mapping for '{service_name}' service from '{ports['published']}' to '{port}'.")
                             ports['published'] = port 
 
-                    # Add devices (if they're present)
-                    # Will check the devices available on the system. If they don't match what's provided in the .avtoolbox.yml file, ignore them
-					# https://stackoverflow.com/a/68402011
-                    for device_service_name, device_list in custom_config["optional_devices"].items():
-                        if device_service_name == service_name:
-                            for device in parse_devices(device_list):
-                                if os.path.exists(device["PathOnHost"]):
-                                    if 'devices' in service:
-                                        service['devices'].append(device['Original'])
-                                    else:
-                                        service['devices'] = [device['Original']]
+                    # Add custom cli arguments
+                    for arg_name, service_dict in custom_config["custom_cli_arguments"].items():
+                        if arg_name in unknown_args and service_name in service_dict:
+                            service.update(_merge_dictionaries(service, service_dict[service_name]))
 
                 # Rewrite with the parsed config
                 with open(root / "docker-compose.yml", "w") as yaml_file:
@@ -264,6 +262,12 @@ def _run_env(args):
                 shellcmd = env.split("USERSHELLPATH=")[1].split('\n')[0]
 
                 client.run("exec", "dev", exec_cmd=shellcmd, *args.args)
+
+            if args.run:
+                LOGGER.info(f"Running...")
+
+                client.run("run", "chrono", *args.args, no_services=True)
+
         except DockerException as e:
             LOGGER.fatal(e)
             if e.stderr:
@@ -295,8 +299,9 @@ def _init(subparser):
     subparser.add_argument("-u", "--up", action="store_true", help="Spin up the env.", default=False)
     subparser.add_argument("-d", "--down", action="store_true", help="Tear down the env.", default=False)
     subparser.add_argument("-a", "--attach", action="store_true", help="Attach to the env.", default=False)
+    subparser.add_argument("-r", "--run", action="store_true", help="Run a command in the provided service. Only one service may be provided. No other arguments may be called.", default=False)
+    subparser.add_argument("-s", "--services", nargs='+', help="The services to use. Defaults to 'all' or whatever 'default_services' is set to in .avtoolbox.yml. 'dev' or 'all' is required for the 'attach' argument. If 'all' is passed, all the services are used.", default=None)
     subparser.add_argument("--keep-yml", action="store_true", help="Don't delete the generated docker-compose file.", default=False)
-    subparser.add_argument("--services", nargs='+', help="The services to use. Defaults to 'all' or whatever 'default_services' is set to in .avtoolbox.yml. 'dev' or 'all' is required for the 'attach' argument. If 'all' is passed, all the services are used.", default=None)
     subparser.add_argument("--args", nargs=argparse.REMAINDER, help="Additional arguments to pass to the docker compose command. No logic is done on the args, the docker command will error out if there is a problem.", default=[])
     subparser.set_defaults(cmd=_run_env)
 
